@@ -1,4 +1,4 @@
-/** Copyright John Reid 2011
+/** Copyright John Reid 2011, 2012
  *
  * \file
  * \brief Defines the background model type for STEME algorithm.
@@ -8,6 +8,7 @@
 #ifndef STEME_JR_13AUG2011_BACKGROUND_MODEL_H_
 #define STEME_JR_13AUG2011_BACKGROUND_MODEL_H_
 
+#include <steme/seqan_types.h>
 #include <steme/markov.h>
 
 #include <boost/function_output_iterator.hpp>
@@ -71,12 +72,17 @@ struct wmer_likelihoods
 
 
 /**
- * A background model for a particular motif width, W
+ * A background model for a particular motif width, W. This model stores the log likelihoods for each W-mer so
+ * is not suitable when there are many W-mers, e.g. when the indexed text is a genome. This class also optionally
+ * stores the log likelihood for each occurrence of a W-mer.
  */
 struct bg_model
 {
     typedef boost::shared_ptr< bg_model >  ptr;  ///< Pointer.
 
+    /// An empty partial evaluation
+    struct partial_evaluation_t { };
+
     wmer_likelihoods               wmer_LLs;       ///< Log likelihoods of the W-mers.
     const likelihoods_vec_vec_t * base_LLs;       ///< Log likelihoods per-base.
     zero_order_frequencies         freqs;          ///< The 0-order frequencies.
@@ -97,10 +103,26 @@ struct bg_model
     }
 
 
+    /// Return a null partial evaluation.
+    static
+    partial_evaluation_t
+    empty_partial_evaluation() {
+        return partial_evaluation_t();
+    }
+
+
+    /// Do nothing as we have no partial evaluation.
+    template< typename Prefix >
+    partial_evaluation_t
+    update_partial_eval( Prefix, partial_evaluation_t, size_t ) {
+        return partial_evaluation_t();
+    }
+
+
     /// Return the minimum log likelihood of any W-mer that starts with the prefix represented by this iterator.
     template< typename TopDownIt >
     double
-    get_min_log_likelihood( TopDownIt it ) const {
+    get_min_log_likelihood( TopDownIt it, partial_evaluation_t, size_t, size_t ) const {
         return wmer_LLs.get_min_log_likelihood( it );
     }
 
@@ -135,7 +157,6 @@ struct bg_model
 
 
 
-
 /**
  * Calculates likelihoods of W-mers in the data given a model. Uses CRTP C++ idiom.
  * The Derived class is the model.
@@ -157,7 +178,7 @@ struct likelihood_calculator
 	calculate( Data & _data ) {
 		using namespace seqan;
 		resizeVertexMap( _data.index, likelihoods.min_log );
-		visit_node( _data, static_cast< Derived * >( this )->index_root( _data ) );
+		visit_node( _data, static_cast< Derived * >( this )->index_root( _data ), 0 );
 	}
 
 
@@ -180,12 +201,12 @@ struct likelihood_calculator
 		typename It
 	>
 	double
-	visit_node( Data & _data, It it ) {
+	visit_node( Data & _data, It it, size_t last_length ) {
 		using namespace seqan;
 
 		MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
 
-		static_cast< Derived * >( this )->entered_node( it );
+		static_cast< Derived * >( this )->entered_node( it, last_length );
 
 		// index into our bounds.
 		typename VertexDescriptor<
@@ -195,7 +216,7 @@ struct likelihood_calculator
 		double min_ll = 0.; // Return a high bound on the log likelihood if we don't find a lower one.
 
 		// how far down the tree are we?
-		const size_t w = std::min( _data.get_rep_length( it ), likelihoods.W );
+		const size_t w = std::min( repLength( it ), likelihoods.W );
 
 		// did we get to the bottom?
 		if( likelihoods.W == w ) {
@@ -213,9 +234,9 @@ struct likelihood_calculator
 
 			// continue : so go down and across
 			if( goDown( it ) ) {
-				min_ll = visit_node( _data, it );
+				min_ll = visit_node( _data, it, w );
 				while( goRight( it ) ) {
-					min_ll = std::min( min_ll, visit_node( _data, it ) );
+					min_ll = std::min( min_ll, visit_node( _data, it, w ) );
 				}
 			}
 		}
@@ -224,7 +245,7 @@ struct likelihood_calculator
 		MYRRH_ASSERT( min_ll <= 0. ); // perhaps not for empty tree...
 		seqan::assignProperty( likelihoods.min_log, v, min_ll );
 
-        static_cast< Derived * >( this )->left_node( it );
+        static_cast< Derived * >( this )->left_node( it, last_length );
 
 		return min_ll;
 	}
@@ -575,7 +596,7 @@ struct markov_0_order_likelihoods_calculator
      */
     template< typename It >
     void
-    entered_node( It it ) {
+    entered_node( It it, size_t last_length ) {
     }
 
     /**
@@ -583,7 +604,7 @@ struct markov_0_order_likelihoods_calculator
      */
     template< typename It >
     void
-    left_node( It it ) {
+    left_node( It it, size_t last_length ) {
     }
 
 
@@ -648,7 +669,7 @@ struct base_to_wmer_likelihoods_calculator
      */
     template< typename It >
     void
-    entered_node( It it ) {
+    entered_node( It it, size_t last_length ) {
     }
 
     /**
@@ -656,7 +677,7 @@ struct base_to_wmer_likelihoods_calculator
      */
     template< typename It >
     void
-    left_node( It it ) {
+    left_node( It it, size_t last_length ) {
     }
 
 
@@ -725,30 +746,21 @@ struct markov_wmer_likelihoods_calculator
     }
 
 
-    /** The iterator used for the descent. */
-    template< typename Data >
-    typename Data::top_down_history_it
-    index_root( Data & _data ) {
-        return typename Data::top_down_history_it( _data.index );
-    }
-
-
     /**
      * Called by base class
      */
     template< typename It >
     void
-    entered_node( It it ) {
+    entered_node( It it, size_t last_length ) {
         using namespace seqan;
         MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
 
         // work out where we are
         const size_t length = std::min( base_t::likelihoods.W, repLength( it ) );
-        const size_t offset = repLength( it ) - parentEdgeLength( it );
 
         // resize evaluations
-        MYRRH_ASSERT( current_evals.size() >= offset );
-        current_evals.resize( offset );
+        MYRRH_ASSERT( current_evals.size() >= last_length );
+        current_evals.resize( last_length );
 
         // evaluate part of the representative
         typedef typename Container< It >::Type IndexT;
@@ -757,7 +769,7 @@ struct markov_wmer_likelihoods_calculator
         range_it begin = boost::begin( representative( it ) );
         model.evaluate_in_context(
             begin,
-            begin + offset,
+            begin + last_length,
             begin + length,
             boost::make_function_output_iterator( make_push_back_cumulative( current_evals ) )
         );
@@ -768,7 +780,7 @@ struct markov_wmer_likelihoods_calculator
      */
     template< typename It >
     void
-    left_node( It it ) {
+    left_node( It it, size_t last_length ) {
     }
 
     /**
