@@ -19,6 +19,14 @@ namespace steme {
 
 
 
+/// forward decl
+template< typename Spec = default_spec >
+struct data;
+
+
+
+
+
 
 /**
  * An abstract base class for a base-pair resolution PSP.
@@ -186,14 +194,6 @@ struct psp {
 
 
 
-/// forward decl
-template< typename Spec = default_spec >
-struct data;
-
-
-
-
-
 /**
  * Class that calculates model's priors from the base resolution priors in the data.
  *
@@ -220,13 +220,13 @@ struct model_priors_calculator {
     /// Calculate model's priors from the base resolution priors in the data.
     void
     calculate_weights() {
-        visit_node( top_down_history_it( _data.index ) );
+        visit_node( top_down_it( _data.index ) );
     }
 
 protected:
     /// Visit a node to calculate its weight
     double
-    visit_node( top_down_history_it it ) {
+    visit_node( top_down_it it ) {
         using namespace seqan;
 
         MS_DEBUG_STRING_VAR( w_mer, representative( it ) );
@@ -238,7 +238,7 @@ protected:
         double min_g;
 
         // if we haven't reached depth W in tree, visit the node's children and update the maximum weight from them
-        const size_t w = _data.get_rep_length( it );
+        const size_t w = repLength( it );
         if( w < W ) {
             if( goDown( it ) ) {
                 min_g = visit_node( it );
@@ -457,9 +457,81 @@ public:
 };
 
 
+/**
+ * Holds a concatenated string of all sequences to make access to W-mers quick.
+ */
+template< typename Derived, typename Spec >
+struct wmer_concatenated_string_policy {
+    STEME_TYPEDEF_SEQAN_TYPES( Spec )
+    string_t        concatenated_seqs;  ///< A concatenation of all the strings in the index's text (this is a copy of the strings).
+
+protected:
+    void initialise() {
+        // copy all the strings into one long string
+        // this means get_W_mer() can be much more efficient.
+        BOOST_FOREACH( string_t & seq, static_cast< Derived * >( this )->get_text() ) {
+            seqan::append( concatenated_seqs, seq );
+        }
+    }
+
+public:
+    /**
+     * Gets the W-mer at the given global position.
+     *
+     * Is faster than the previous method that used posLocalize to get to a local position first.
+     */
+    infix_t
+    get_W_mer( size_t W, int pos ) {
+        return seqan::infixWithLength( concatenated_seqs, pos, W );
+    }
+};
+
+
+/**
+ * Accesses W-mers by converting a global position into a seq # and offset. Does not need to store a
+ * concatenated string.
+ */
+template< typename Derived, typename Spec >
+struct wmer_simple_policy {
+    STEME_TYPEDEF_SEQAN_TYPES( Spec )
+
+protected:
+    /** No initialisation to do. */
+    void initialise() { }
+
+public:
+    /**
+     * Get the W-mer that starts at the given index.
+     */
+    infix_t
+    get_W_mer( size_t W, int pos ) {
+        using namespace seqan;
+        id_t id;
+        int offset;
+        boost::tie( id, offset ) = static_cast< Derived * >( this )->pos_localise( pos );
+#ifdef MS_DO_DEBUG_CHECKS
+        if( offset < 0 || length( getValueById( static_cast< Derived * >( this )->get_text(), id ) ) < offset + W ) {
+            throw std::logic_error(
+                MS_MAKE_STRING(
+                    "No W-mer at given position (" << pos << "). That sequence is too short. Id="
+                    << id << ", offset=" << offset
+                )
+            );
+        }
+#endif //MS_DO_DEBUG_CHECKS
+        return infix( getValueById( static_cast< Derived * >( this )->get_text(), id ), offset, offset+W );
+    }
+};
 
 
 
+/**
+ * Chooses the policy for how the data class handles retrieving W-mers.
+ */
+template< typename Spec >
+struct choose_wmer_policy {
+    typedef wmer_simple_policy< data< Spec >, Spec > type; ///< choose a concatenated string policy by default.
+};
 
 
 
@@ -473,24 +545,23 @@ public:
 template< typename Spec >
 struct data
 : boost::noncopyable
+, choose_wmer_policy< Spec >::type
 {
-
+    typedef typename choose_wmer_policy< Spec >::type     wmer_policy_t;         ///< Mixin class that handles how to get W-mers.
 	typedef data< Spec >                                   self_t;                ///< Self type.
 	typedef boost::shared_ptr< self_t >                    ptr;                   ///< Pointer.
-	typedef seqan::String< size_t >                        size_vec;              ///< Vector of sizes.
-	typedef std::map< size_t, size_t >                     _counts_t;             ///< Type to store counts for each width/depth.
-	typedef std::map< size_t, typename psp< Spec >::ptr >  priors_cache_t;        ///< Type to cache priors for specific widths.
-	STEME_TYPEDEF_SEQAN_TYPES( Spec )
+	typedef unsigned char                                first_unknown_t;       ///< Type to store position of first unknown.
+	typedef seqan::String< first_unknown_t >               first_unknown_vec;     ///< Vector of sizes.
+	typedef std::vector< size_t >                          _counts_t;             ///< Type to store counts for each width/depth.
+	typedef std::map< size_t, typename psp< Spec >::ptr >  priors_cache_t;      ///< Type to cache priors for specific widths.
+    STEME_TYPEDEF_SEQAN_TYPES( Spec )
 
 	index_t &                            index;              ///< The index over our sequences.
 	const raw_text_t &                   raw_text;           ///< A concatenation of all the strings in the index's text (this view is provided by the index).
-	string_t                             concatenated_seqs;  ///< A concatenation of all the strings in the index's text (this is a copy of the strings).
 	size_t                               N;                  ///< The number of bases in the sequences (i.e. number of global positions).
-	//size_vec                             rep_lengths;        ///< The length of the representative of each node in the suffix tree.
-	size_vec                             first_unknown;      ///< Stores index of first position of unknown base in representative of each node.
-    _counts_t                            node_counts;        ///< How many nodes at each depth (excepting W-mers with unknown bases).
-    _counts_t                            occurrence_counts;  ///< How many occurrences at each depth (excepting W-mers with unknown bases).
+	first_unknown_vec                    first_unknown;      ///< Stores index of first position of unknown base in representative of each node.
     size_t                               max_W;              ///< Maximum width to consider for counts.
+    _counts_t                            occurrence_counts;  ///< How many occurrences at each depth (excepting W-mers with unknown bases).
     bool                                 using_dense;        ///< Are we using dense priors or not?
 	typename base_resolution_psp::ptr    _base_prior;        ///< Position-specific prior for each base. Prior probability base is not available for binding.
 	priors_cache_t                       _priors_cache;      ///< Caches prior for each width.
@@ -502,6 +573,7 @@ struct data
 		, raw_text( seqan::getFibre( index, seqan::EsaRawText() ) )
 		, N( seqan::length( index ) )
 	    , max_W( max_W )
+	    , occurrence_counts( max_W + 1 )
 	    , using_dense( using_dense )
 		, _base_prior(
 			using_dense
@@ -509,12 +581,10 @@ struct data
 				: base_resolution_psp::ptr( new sparse_base_resolution_psp( index ) )
 		)
 	{
-		// copy all the strings into one long string
-		// this means get_W_mer() can be much more efficient.
-		BOOST_FOREACH( string_t & seq, get_text() ) {
-			seqan::append( concatenated_seqs, seq );
-		}
-
+	    if( max_W > std::numeric_limits< first_unknown_t >::max() ) {
+	        throw std::invalid_argument( "max_W too big for first_unknown vector." );
+	    }
+	    wmer_policy_t::initialise();
 		preprocess_index();
 	}
 
@@ -523,60 +593,50 @@ struct data
 	void
 	preprocess_index() {
         seqan::resizeVertexMap( index, first_unknown );
-        //seqan::resizeVertexMap( index, rep_lengths );
-		visit_node( top_down_history_it( index ), 0, std::numeric_limits< size_t >::max() );
+		visit_node( top_down_it( index ), 0, std::numeric_limits< size_t >::max() );
 	}
 
 
 	/// Visit a node to calculate the length of its representative.
 	void
-	visit_node( top_down_history_it it, size_t last_length, size_t first_unknown_index ) {
+	visit_node( top_down_it it, size_t last_length, size_t first_unknown_index ) {
 		using namespace seqan;
 
-		// get the parent edge.
-		const parent_edge_label_t & parent_edge = parentEdgeLabel( it );
-        const size_t parent_edge_len = length( parent_edge );
-        const size_t this_length = last_length + parent_edge_len;
+		// get the new length.
+        const size_t this_length = repLength( it );
 
         // index into our property vectors
-        const typename Value< top_down_history_it >::Type idx = value( it );
+        const typename Value< top_down_it >::Type idx = value( it );
 
-        // look for an unknown base, if we didn't have one already
-        if( std::numeric_limits< size_t >::max() == first_unknown_index ) {
-            const int first_unknown_in_parent = find_first_unknown( parent_edge );
-            if( -1 != first_unknown_in_parent ) {
-                first_unknown_index = first_unknown_in_parent + last_length;
-            }
+        // look for an unknown base
+        const int first_unknown_in_parent =
+            find_first_unknown(
+                infix(
+                    representative( it ),
+                    last_length,
+                    max_W ? std::min( max_W, this_length ) : this_length
+                )
+            );
+        if( -1 != first_unknown_in_parent ) {
+            first_unknown_index = first_unknown_in_parent + last_length;
         }
         assignProperty( first_unknown, idx, first_unknown_index );
 
-        // update the counts of nodes at each depth until we reach the end of this representative, the maximum W or the first
+        // update the counts of occurrences at each depth until we reach the end of this representative, the maximum W or the first
         // unknown
+        const size_t end = std::min( this_length + 1, first_unknown_index );
+        if( 0 == max_W && end > occurrence_counts.size() ) {
+            occurrence_counts.resize( end, 0 ); // resize if needed
+        }
         const size_t num_known = countOccurrences( it );
-        for( size_t w = last_length + 1; ( ! max_W || max_W >= w ) && std::min( this_length + 1, first_unknown_index ) > w; ++w ) {
-            // add the node count
-            typename _counts_t::iterator i = node_counts.find( w );
-            if( node_counts.end() == i ) {
-                bool inserted;
-                boost::tie( i, inserted ) = node_counts.insert( typename _counts_t::value_type( w, 0 ) );
-            }
-            ++( i->second );
-
+        for( size_t w = last_length + 1; ( ! max_W || max_W >= w ) && end > w; ++w ) {
             // add the occurrence count
-            i = occurrence_counts.find( w );
-            if( occurrence_counts.end() == i ) {
-                bool inserted;
-                boost::tie( i, inserted ) = occurrence_counts.insert( typename _counts_t::value_type( w, 0 ) );
-            }
-            i->second += num_known;
+            BOOST_ASSERT( w < occurrence_counts.size() );
+            occurrence_counts[ w ] += num_known;
         }
 
-		// store the representative length
-        BOOST_ASSERT( this_length == repLength( it ) );
-		// assignProperty( rep_lengths, idx, this_length );
-
-		// only go deeper if we haven't reached maximum depth already
-		if( ! max_W || max_W > this_length ) {
+        // only go deeper if we haven't reached maximum depth already and we don't have an unknown base
+        if( ( ! max_W || max_W > this_length ) && std::numeric_limits< size_t >::max() == first_unknown_index ) {
             // visit the node's children
             if( goDown( it ) ) {
                 visit_node( it, this_length, first_unknown_index );
@@ -638,19 +698,10 @@ struct data
     }
 
 
-    /// How many known nodes at the given depth?
-    size_t
-    get_node_count( size_t W ) {
-        typename _counts_t::iterator i = node_counts.find( W );
-        return node_counts.end() == i ? 0 : i->second;
-    }
-
-
     /// How many known occurrences at the given depth?
     size_t
     get_occurrence_count( size_t W ) {
-        typename _counts_t::iterator i = occurrence_counts.find( W );
-        return occurrence_counts.end() == i ? 0 : i->second;
+        return W > occurrence_counts.size() ? 0 : occurrence_counts[ W ];
     }
 
 
@@ -667,7 +718,6 @@ struct data
 	size_t
 	get_rep_length( top_down_it it ) {
 	    return seqan::repLength( it );
-		//return seqan::getProperty( rep_lengths, value( it ) );
 	}
 
 
@@ -721,43 +771,6 @@ struct data
 	}
 
 
-	/**
-	 * Gets the W-mer at the given global position.
-	 *
-	 * Is faster than the previous method that used posLocalize to get to a local position first.
-	 */
-	infix_t
-	get_W_mer_new( size_t W, int pos ) {
-		using namespace seqan;
-		// check has same result as old method to get W-mer
-		MYRRH_ASSERT( get_W_mer( W, pos ) == infix( concatenated_seqs, pos, pos+W ) );
-		return infixWithLength( concatenated_seqs, pos, W );
-	}
-
-
-	/**
-	 * Get the W-mer that starts at the given index.
-	 */
-	infix_t
-	get_W_mer( size_t W, int pos ) {
-		using namespace seqan;
-		id_t id;
-		int offset;
-		boost::tie( id, offset ) = pos_localise( pos );
-#ifdef MS_DO_DEBUG_CHECKS
-		if( offset < 0 || length( getValueById( get_text(), id ) ) < offset + W ) {
-				throw std::logic_error(
-					MS_MAKE_STRING(
-						"No W-mer at given position (" << pos << "). That sequence is too short. Id="
-						<< id << ", offset=" << offset
-					)
-				);
-		}
-#endif //MS_DO_DEBUG_CHECKS
-		return infix( getValueById( get_text(), id ), offset, offset+W );
-	}		/** Get the text underlying the index. */
-
-
 	/// Get the index's text.
 	text_t &
 	get_text() {
