@@ -89,6 +89,12 @@ struct EM_descender
 	, Z_sparse( _data.N, _data.N / 3 )
 	, Z_dense( _data.N )
 	, large_Z_threshold( 1. / base_t::W )
+	, expected_sites( 0 )
+	, LL( 0. )
+    , log_lambda_ratio( base_t::_model._prior.lambda.get_log_lambda_ratio() )
+    , num_Z_large( 0 )
+    , num_Z_non_zero( 0 )
+    , num_Z_normalised( 0 )
 	{
 	}
 
@@ -103,7 +109,7 @@ struct EM_descender
 	/** Do one iteration. */
 	double
 	do_iteration() {
-		tree_descender< EM_descender >::descend_tree();
+	    base_t::descend_tree();
 		return expected_sites;
 	}
 
