@@ -120,9 +120,9 @@ struct model
 {
 
 	typedef boost::shared_ptr< model >                     ptr;                          ///< A pointer to a model.
-    typedef data< Spec >                                   data_t;                       ///< The data type.
-	typedef typename binding_model_meta< Spec >::type      bs_model_t;                   ///< The binding site model.
-	typedef typename background_model_meta< Spec >::type   bg_model_t;                   ///< The background model.
+    typedef data< Spec >                                  data_t;                       ///< The data type.
+	typedef typename binding_model_meta< Spec >::type    bs_model_t;                   ///< The binding site model.
+	typedef typename background_model_meta< Spec >::type bg_model_t;                   ///< The background model.
 	STEME_TYPEDEF_SEQAN_TYPES( Spec )
 
 
@@ -182,8 +182,8 @@ public:
         const int offset = seqan::getSeqOffset( global_pos, _data.get_string_set_limits() );
 	    const double log_p_binding =
             rev_comp
-                ? w_mer_binding_log_likelihood< true  >( bs, _data.get_W_mer_new( W(), global_pos ) )
-                : w_mer_binding_log_likelihood< false >( bs, _data.get_W_mer_new( W(), global_pos ) )
+                ? w_mer_binding_log_likelihood< true  >( bs, _data.get_W_mer( W(), global_pos ) )
+                : w_mer_binding_log_likelihood< false >( bs, _data.get_W_mer( W(), global_pos ) )
                 ;
 	    const double log_p_background = bg.wmer_log_likelihood( seq, offset );
 	    const double lambda = _prior.lambda.get_lambda();
@@ -200,7 +200,7 @@ public:
 	 */
 	typename seqan::Infix< string_t >::Type
 	get_W_mer( int pos ) {
-		return _data.get_W_mer_new( W(), pos );
+		return _data.get_W_mer( W(), pos );
 	}
 
 
