@@ -17,8 +17,6 @@
 
 #include <boost/python/make_function.hpp>
 
-#include <steme/seqan_types.h>
-
 #include <myrrh/python/tuple.h>
 #include <myrrh/python/multi_array_to_numpy.h>
 #include <myrrh/python/boost_function.h>
@@ -26,7 +24,32 @@
 #include <indexing_suite/vector.hpp>
 #include <indexing_suite/map.hpp>
 
-#define EXPOSED_STEME_SPEC ::steme::default_spec
+#define STEME_INDEX_DEFAULT_TYPE 0
+#define STEME_INDEX_GENOME_TYPE 1
+
+
+
+// Choose default specification if not defined
+#ifndef STEME_INDEX_MODULE_TYPE
+# define STEME_INDEX_MODULE_TYPE STEME_INDEX_DEFAULT_TYPE
+#endif //STEME_INDEX_MODULE_TYPE
+
+
+
+#if STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+# define STEME_INDEX_MODULE_SPEC ::steme::default_spec
+# include <steme/spec_default.h>
+
+#elif STEME_INDEX_MODULE_TYPE == STEME_INDEX_GENOME_TYPE
+# define STEME_INDEX_MODULE_SPEC ::steme::genome_spec
+# include <steme/spec_genome.h>
+
+#else
+# error Unknown index module type
+
+#endif //STEME_INDEX_MODULE_SPEC
+
+
 
 
 // choose maximum Markov model order if not decided already.
@@ -48,12 +71,12 @@ template< typename Spec > struct data;
 namespace python {
 
 
-typedef steme_seqan_types< EXPOSED_STEME_SPEC > seqan_types_t;
+typedef steme_seqan_types< STEME_INDEX_MODULE_SPEC > seqan_types_t;
 
 
 /// The model.
 template< typename Exposed >
-model< EXPOSED_STEME_SPEC > &
+model< STEME_INDEX_MODULE_SPEC > &
 get_model( Exposed & e ) {
 	return e._model;
 }
@@ -61,7 +84,7 @@ get_model( Exposed & e ) {
 
 /// The data.
 template< typename Exposed >
-data< EXPOSED_STEME_SPEC > &
+data< STEME_INDEX_MODULE_SPEC > &
 get_data( Exposed & e ) {
 	return e._data;
 }
