@@ -8,7 +8,7 @@ Models for STEME web app.
 """
 
 
-import uuid
+from uuid import uuid4 
 from datetime import datetime
 
 try:
@@ -33,7 +33,7 @@ class Job(db.Model):
     completed = db.Column(db.Boolean)
 
     def __init__(self, name=None):
-        self.uuid = str(uuid.uuid4())
+        self.uuid = str(uuid4())
         self.pid = 0
         self.name = name
         self.completed = False
