@@ -1587,7 +1587,7 @@ class MotifFinder(object):
             [start_result.log_E_value for start_result in start_results],
         )
         P.xlim((-2,102))
-        P.xlabel('% coverage of start instances')
+        P.xlabel('\% coverage of start instances')
         P.ylabel('log E-value')
         filename = os.path.join(self._output_dir_for_motif(), 'coverage-vs-E-value')
         logger.info('Saving coverage vs. E-value graph to %s', filename)
@@ -1633,7 +1633,7 @@ class MotifFinder(object):
                     
         logger.info('Ran EM on %d starts', len(start_results))
         self._plot_num_sites_graph(start_results)
-        # self._plot_coverage_vs_E_value(start_results) # don't plot as sometimes causes exception (need to investigate)
+        self._plot_coverage_vs_E_value(start_results) # don't plot as sometimes causes exception (need to investigate)
         
         
         #
