@@ -7,7 +7,6 @@ Code to choose whether to import debug or release build of stempy module.
 """
 
 import sys, logging
-
 logger = logging.getLogger(__name__)
 
 #
@@ -16,18 +15,17 @@ logger = logging.getLogger(__name__)
 _python_debug_build = hasattr(sys, "gettotalrefcount") # only available in python debug build
 if _python_debug_build:
 
-    logger.info('Loading debug build of STEME C++-python interface')
     from ._debug_build import *
-    from ._debug_build import _dummy_fn, _debug, _using_old_bg_model
+    from ._debug_build import _dummy_fn, _debug, _has_google_profiler
     from . import _debug_build as S
+    if _has_google_profiler:
+        from _debug_build import __google_profiler_start, __google_profiler_stop
 
 else:
     
-    logger.info('Loading release build of STEME C++-python interface')
     from ._release_build import *
-    from ._release_build import _dummy_fn, _debug, _using_old_bg_model, _has_google_profiler
+    from ._release_build import _dummy_fn, _debug, _has_google_profiler
     from . import _release_build as S
-    
     if _has_google_profiler:
         from _release_build import __google_profiler_start, __google_profiler_stop
         
